import { Pab, Result } from "jalhyd";

import { FormulaireDefinition } from "./form-definition";
import { PabResults } from "../../results/pab-results";
import { NgParameter } from "../elements/ngparam";
import { longestVarNgParam } from "../../util";
import { CalculatorResults } from "../../results/calculator-results";

/**
 * Formulaire pour les passes à bassins, inspiré du formulaire
 * pour les structures en parallèle
 */
export class FormulairePab extends FormulaireDefinition {

    protected _pabResults: PabResults;

    constructor() {
        super();
        this._pabResults = new PabResults();
    }

    public get pabNub(): Pab {
        return this.currentNub as Pab;
    }

    protected compute() {
        this.runNubCalc(this.currentNub);
        // reset variable index to avoid trying to access an index > 0 when nothing varies
        const pabr = this.pabResults;
        pabr.variableIndex = 0;

        this.reaffectResultComponents();
    }

    protected reaffectResultComponents() {
        const pab: Pab = (this.currentNub as Pab);
        const computedParam: NgParameter = this.getComputedParameter();
        const varParams: NgParameter[] = this.getVariatedParameters();

        // résultat de calcul de la passe à bassins
        const pabr = this.pabResults;
        pabr.calculatedParameter = computedParam;
        pabr.result = pab.result;

        // résultat de chaque cloison
        const cr: Result[] = [];
        for (const c of pab.children) {
            cr.push(c.result);
        }
        pabr.cloisonsResults = cr;
        // résultat de la cloison aval
        pabr.cloisonAvalResults = pab.downWall.result;

        // cote aval de la passe
        pabr.Z2 = [];
        if (varParams.length > 0) {
            // find longest list
            const lvp = longestVarNgParam(varParams);
            const longest = lvp.size;
            // get extended values lists for Z2
            if (pab.prms.Z2.hasMultipleValues) {
                const iter = pab.prms.Z2.getExtendedValuesIterator(longest);
                while (iter.hasNext) {
                    const nv = iter.next();
                    pabr.Z2.push(nv.value);
                }
            } else {
                for (let i = 0; i < longest; i++) {
                    pabr.Z2.push(pab.prms.Z2.v);
                }
            }
        } else {
            pabr.Z2 = [ pab.prms.Z2.singleValue ];
        }

        if (varParams) {
            pabr.variatedParameters = varParams;
        }
    }

    public get pabResults() {
        return this._pabResults;
    }

    public resetResults() {
        this._pabResults.reset();
    }

    public get results(): CalculatorResults[] {
        // ensure help links are propagated
        this._pabResults.helpLinks = this.helpLinks;
        return [ this._pabResults ];
    }

    public get hasResults(): boolean {
        return this._pabResults.hasResults;
    }
}
